package xn.update;

import android.net.Uri;
import android.util.Log;

import androidx.annotation.WorkerThread;

import com.blankj.utilcode.util.AppUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.SPUtils;
import com.blankj.utilcode.util.ShellUtils;
import com.hikvision.dmb.system.InfoSystemApi;
import com.hikvision.dmb.util.InfoUtilApi;

import okhttp3.HttpUrl;
import xn.update.constant.AppConstant;

public enum Tool {
    INSTANCE;

    Tool() {
        InfoUtilApi.getRoot();
    }

    public void reStartApp() {
        exitApp();
        InfoSystemApi.execCommand("monkey -p " + AppUtils.getAppPackageName() + " -c android.intent.category.LAUNCHER 1");
    }

    public void exitApp() {
        InfoSystemApi.execCommand("am force-stop " + AppUtils.getAppPackageName());
    }

    public void reStartApp(String packageName) {
        exitApp(packageName);
        InfoSystemApi.execCommand("monkey -p " + packageName + " -c android.intent.category.LAUNCHER 1");
    }

    public void exitApp(String packageName) {
        InfoSystemApi.execCommand("am force-stop " + packageName);
    }

    public void openSetting() {
        openApp("display.interactive.hsetting");
    }

    public void openFileBrowser() {
        openApp("display.interactive.filebrowser");
    }

    public void openApp(String packageName) {
        InfoSystemApi.execCommand("monkey -p " + packageName + " -c android.intent.category.LAUNCHER 1");
    }

    public ShellUtils.CommandResult stopApp(String packageName) {
        return ShellUtils.execCmd("am force-stop " + packageName, true);
    }


    public int cmd(String cmd) {
        int result = InfoSystemApi.execCommand(cmd);
        LogUtils.d(cmd, result);
        return result;
    }

    public String getSerialNumber() {
        return "EBOARD_" + InfoSystemApi.getSerialNumber();
    }

    public HttpUrl getBaseUrl() {
        return HttpUrl.get(SPUtils.getInstance().getString(AppConstant.BASE_URL, "http://172.16.0.65/api/"));
    }

    public String checkUrl(String url) {
        try {
            String uriStr = url;
            if (!url.matches("https?://.*") && !url.matches("http?://.*")) {
                uriStr = Uri.parse(getBaseUrl() + url).toString();
            }
            return uriStr;
        } catch (Exception e) {
            LogUtils.e(Log.getStackTraceString(e));
            return url;
        }
    }

    public void reboot() {
        InfoSystemApi.reboot();
    }
}
