package xn.update.http;


import com.blankj.utilcode.util.AppUtils;
import com.blankj.utilcode.util.ShellUtils;
import com.hikvision.dmb.EthernetConfig;
import com.hikvision.dmb.network.InfoNetworkApi;
import com.hikvision.dmb.system.InfoSystemApi;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import okhttp3.Response;
import xn.update.Tool;

public enum HttpTool {
    INSTANCE;

    public Response terminalAuth() throws IOException, JSONException {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("deviceCode", Tool.INSTANCE.getSerialNumber());
        jsonObject.put("code", "aio_infobord");
        return OkHttpUtils.INSTANCE.postSync(Tool.INSTANCE.getBaseUrl() + "terminal/authorize", jsonObject.toString());
    }

    public Response update() throws IOException, JSONException {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("deviceCode", Tool.INSTANCE.getSerialNumber());
        return OkHttpUtils.INSTANCE.postSync(Tool.INSTANCE.getBaseUrl() + "terminal/upgrade/device/task", jsonObject.toString());
    }

    public void updateCallBack(String taskId, String deviceCode, boolean isDownload, boolean isInstall) throws IOException, JSONException {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("taskId", taskId);
        jsonObject.put("deviceCode", deviceCode);
        if (isDownload) {
            jsonObject.put("status", "downloaded");
        }
        if (isInstall) {
            jsonObject.put("status", "success");
        }
        OkHttpUtils.INSTANCE.postSync(Tool.INSTANCE.getBaseUrl() + "terminal/upgrade/device/callback", jsonObject.toString());
    }

    /**
     * category    1:系统参数 2:公共配置
     * configType  1:基础配置 2: 管控一体机 3:化学品终端 4:小程序配置 5:开发配置 6:首页配置
     */
    public Response getConfig() throws JSONException, IOException {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("category", 2);
        jsonObject.put("configType", 5);
        return OkHttpUtils.INSTANCE.postSync(Tool.INSTANCE.getBaseUrl() + "terminal/sys/config/info/getConfigByType", jsonObject.toString());
    }

    public Response heartbeat() throws JSONException, IOException {
        JSONObject jsonObject = new JSONObject();
        ShellUtils.CommandResult commandResult = ShellUtils.execCmd("dumpsys activity activities", true);
        BufferedReader reader = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(commandResult.successMsg.getBytes())));
        String line;
        boolean isShow = false;
        while ((line = reader.readLine()) != null) {
            if (line.contains("mResumedActivity") || line.contains("mCurrentFocus")) {
                String[] itemArray = line.split(" ", -1);
                for (String item : itemArray) {
                    if (item.contains("/")) {
                        String packageName = item.substring(0, item.indexOf("/"));
                        isShow = AppUtils.getAppPackageName().equals(packageName);
                        break;
                    }
                }
                break;
            }
        }

        boolean adbEnabled = (1 == InfoSystemApi.getAdbStatus());
        EthernetConfig ethernetConfig = InfoNetworkApi.getEthernetConfig();
        // Ip
        String ip = ethernetConfig.getIpAddress();
        jsonObject.put("deviceNo", "EBOARD_" + InfoSystemApi.getSerialNumber());
        jsonObject.put("versionName", AppUtils.getAppVersionName());
        jsonObject.put("packageName", AppUtils.getAppPackageName());
        jsonObject.put("isShow", isShow);
        jsonObject.put("adbEnabled", adbEnabled);
        jsonObject.put("ip", ip);
        return OkHttpUtils.INSTANCE.postSync(Tool.INSTANCE.getBaseUrl() + "terminal/machine/upgrade/heartbeat", jsonObject.toString());
    }

}
