package xn.update.http.bean.request;

import java.util.List;

public class DeviceInfo {
    // 状态栏是否开启
    private boolean isStatusBarEnable;
    // 导航栏是否开启
    private boolean isNavBarEnable;
    // 网络配置
    private NetworkInfo networkInfo;
    // 时间配置
    private TimeInfo timeInfo;
    // 系统配置
    private SystemInfo systemInfo;
    // App配置
    private List<AppInfo> appInfoList;


    private static class AppInfo {
        private String versionName;
        private String packageName;
        private boolean isShow;
    }

    private static class TimeInfo {
        // 开机时间  默认-1
        private long onTime;
        // 关机时间 默认-1
        private long offTime;
        /**
         * 门禁点锁状态1
         * true-当前锁处于上电锁门状态
         * false-当前锁处于断电开门状态或参数错误
         */
        private boolean electricLock1;
        /**
         * 门禁点锁状态2
         * true-当前锁处于上电锁门状态
         * false-当前锁处于断电开门状态或参数错误
         */
        private boolean electricLock2;
    }

    private static class SystemInfo {
        /**
         * 设备型号
         * 返回设备型号，获取失败时返回默认值 DS-D60A
         */
        private String deviceType;
        /**
         * 设备序列号
         * 返回设备序列号, 获取失败时返回默认值 40000000
         */
        private String sn;
        /**
         * adb状态
         * 0 代表 ADB 未打开
         * 1 代表 ADB 打开
         */
        private int adbStatus;
        /**
         * 设备箱体温度
         * 返回失败或没有温度传感器,”faild”字符串
         * 成功返回例 37
         */
        private String temperature;
        /**
         * 系统编译版本号
         * 获取失败返回”unknown”字符串
         * 成功返回例 V2.0.0 build 171226
         */
        private String buildDesc;
        /**
         * 系统MCU版本号
         * 获取失败返回 null
         * 成功返回例 V1.0.0 build 171226
         */
        private String mcuVersion;

        /**
         * CPU使用率 当前 Cpu 的使用率，百分比,
         * 获取失败时返回”-1.00”字符串
         */
        private String cpuUsageRate;
        /**
         * GPU使用率 当前 Gpu 的使用率，百分比,
         * 获取失败时返回”-1.00”字符串
         */
        private String gpuUsageRate;
        // 已使用内存大小(单位 MB)
        private int used;
        // 总内存大小(MB)
        private int total;
        // 583.00MB/1.96GB
        private String describe;
        private int describeContents;
        // 获取SDK版本 返回 SDK 版本号，获取失败返回”none”字符串
        private String sdkVersion;
        // 为当前生效的默认 Launcher 应用包名，默认（始终）Launcher 应用修改后该值即时更新。如果当前系统未设置默认（始终）Launcher，则该值为空字符串(“”)。
        private String currentLauncherName;
        // 为持久化文件中保存的默认 Launcher 应用包名，默认 Launcher 应用修改后约 10 秒后该值才会更新，该值仅当系统内存在 2 个及以上Launcher 应用时才会存在，否则为空字符串(“”)。
        private String preferredLauncherName;
    }

    private static class NetworkInfo {
        /**
         * ip
         * 代表有线网络 ip 地址
         * 默认值：”0.0.0.0”
         */
        private String ip;
        /**
         * dhcp
         * 0—代表自动获取网络(默认)
         * 1—代表静态设置网络信息模式
         */
        private int dhcp;
        // dns1 默认值：”0.0.0.0”
        private String dns1;
        // dns2 默认值：”0.0.0.0”
        private String dns2;
        /**
         * mac
         * 代表有线网络 mac 地址，
         * 有线网络未连接时，该属性仍有值，
         * 默认值：”00:00:00:00:00:00”
         */
        private String mac;
        /**
         * route
         * 代表有线网络网关地址
         * 默认值：”0.0.0.0”
         */
        private String route;
        /**
         * subnetMask
         * 代表有线网络子网掩码地址
         * 默认值：”0.0.0.0”
         */
        private String subnetMask;
        // 最优Ip地址   以太网>无线网>3、4G
        private String optimalIp;
    }
}
